@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.3
+#!/usr/local/bin/python2.3
 
 ##############################################################################
 #
@@ -14,7 +14,7 @@
 #
 ##############################################################################
 """
-test.py [-abcdDfgGhLmprtTuv] [modfilter [testfilter]]
+test.py [-abcCdDfgGhLmprtTuv] [modfilter [testfilter]]
 
 Find and run tests written using the unittest module.
 
@@ -46,7 +46,14 @@ named .testinfo, it will not be searched for tests.  Really.)
     In Python < 2.3 the -q flag is added to the setup.py command
     line.)
 
--c  use pychecker
+-c  
+    Use pychecker
+
+--config-file filename
+    Configure Zope by loading the specified configuration file (zope.conf).
+
+-C filename
+    Shortcut for --config-file filename.
 
 -d
     Instead of the normal test harness, run a debug version which
@@ -79,11 +86,17 @@ named .testinfo, it will not be searched for tests.  Really.)
     of the DEBUG_ flags defined bythe Python gc module.  Multiple options
     can be specified by using "-G OPTION1 -G OPTION2."
 
+--import-testing
+    Import the Testing package to setup the test ZODB.  Useful for running
+    tests that forgot to "import Testing".
+
 --libdir test_root
     Search for tests starting in the specified start directory
     (useful for testing components being developed outside the main
     "src" or "build" trees).
 
+    Note: This directory will be prepended to sys.path!
+
 --keepbytecode
     Do not delete all stale bytecode before running tests
 
@@ -242,15 +255,31 @@ class ImmediateTestResult(unittest._TextTestResult):
         for i in range(n):
             print >>stream, "%6dms" % int(results[i][1] * 1000), results[i][0]
 
-    def _print_traceback(self, msg, err, test, errlist):
-        if self.showAll or self.dots or self._progress:
-            self.stream.writeln("\n")
-            self._lastWidth = 0
-
+    def _handle_problem(self, err, test, errlist):
+        
+        if self._debug:
+            raise err[0], err[1], err[2]
+        
+        if errlist is self.errors:
+            prefix = 'Error'
+        else:
+            prefix = 'Failure'
+            
         tb = "".join(traceback.format_exception(*err))
-        self.stream.writeln(msg)
-        self.stream.writeln(tb)
-        errlist.append((test, tb))
+        
+        if self._progress:
+            self.stream.writeln("\r")
+            self.stream.writeln("%s in test %s" % (prefix,test))
+            self.stream.writeln(tb)
+            self._lastWidth = 0
+        elif self.showAll:
+            self._lastWidth = 0
+            self.stream.writeln(prefix.upper())
+        elif self.dots:
+            self.stream.write(prefix[0])
+            
+        if not self._progress:
+            errlist.append((test, tb))
 
     def startTest(self, test):
         if self._progress:
@@ -288,20 +317,10 @@ class ImmediateTestResult(unittest._TextTestResult):
         return s[:self._maxWidth]
 
     def addError(self, test, err):
-        if self._progress:
-            self.stream.write("\r")
-        if self._debug:
-            raise err[0], err[1], err[2]
-        self._print_traceback("Error in test %s" % test, err,
-                              test, self.errors)
+        self._handle_problem(err, test, self.errors)
 
     def addFailure(self, test, err):
-        if self._progress:
-            self.stream.write("\r")
-        if self._debug:
-            raise err[0], err[1], err[2]
-        self._print_traceback("Failure in test %s" % test, err,
-                              test, self.failures)
+        self._handle_problem(err, test, self.failures)
 
     def printErrors(self):
         if self._progress and not (self.dots or self.showAll):
@@ -349,29 +368,29 @@ class ImmediateTestRunner(unittest.TextTestRunner):
 # setup list of directories to put on the path
 class PathInit:
     def __init__(self, build, libdir=None):
-        # Calculate which directories we're going to add to sys.path, and cd
-        # to the appropriate working directory
-        org_cwd = os.getcwd()
+        # Calculate which directories we're going to add to sys.path.
         self.libdir = "lib/python"
         # Hack sys.path
-        self.cwd = os.getcwd()
-        sys.path.insert(0, os.path.join(self.cwd, self.libdir))
+        self.home = os.path.dirname(os.path.realpath(sys.argv[0]))
+        # test.py lives in $ZOPE_HOME/bin when installed ...
+        dir, file = os.path.split(self.home)
+        if file == 'bin': self.home = dir
+        sys.path.insert(0, os.path.join(self.home, self.libdir))
+        self.cwd = os.path.realpath(os.getcwd())
         # Hack again for external products.
-        global functional
-        kind = functional and "functional" or "unit"
         if libdir:
-            extra = os.path.join(org_cwd, libdir)
-            print "Running %s tests from %s" % (kind, extra)
-            self.libdir = extra
-            sys.path.insert(0, extra)
+            self.libdir = os.path.realpath(os.path.join(self.cwd, libdir))
         else:
-            print "Running %s tests from %s" % (kind, self.cwd)
-        # Make sure functional tests find ftesting.zcml
-        if functional:
-            config_file = 'ftesting.zcml'
-            print "Parsing %s" % config_file
-            from zope.testing.functional import FunctionalTestSetup
-            FunctionalTestSetup(config_file)
+            self.libdir = os.path.realpath(os.path.join(self.cwd, self.libdir))
+        if self.libdir not in sys.path:
+            sys.path.insert(0, self.libdir)
+        # Determine where to look for tests
+        if test_dir:
+            self.testdir = os.path.abspath(os.path.join(self.cwd, test_dir))
+        else:
+            self.testdir = self.libdir
+        kind = functional and "functional" or "unit"
+        print "Running %s tests from %s" % (kind, self.testdir)
 
 def match(rx, s):
     if not rx:
@@ -456,8 +475,7 @@ class TestFileFinder:
 def find_tests(rx):
     global finder
     finder = TestFileFinder(pathinit.libdir)
-    walkdir = test_dir or pathinit.libdir
-    os.path.walk(walkdir, finder.visit, rx)
+    walk_with_symlinks(pathinit.testdir, finder.visit, rx)
     return finder.files
 
 def package_import(modname):
@@ -591,6 +609,24 @@ def runner(files, test_filter, debug):
         else:
             raise
 
+def walk_with_symlinks(path, visit, arg):
+    """Like os.path.walk, but follows symlinks on POSIX systems.
+
+    This could theoretically result in an infinite loop, if you create symlink
+    cycles in your Zope sandbox, so don't do that.
+    """
+    try:
+        names = os.listdir(path)
+    except os.error:
+        return
+    visit(arg, path, names)
+    exceptions = (os.curdir, os.pardir)
+    for name in names:
+        if name not in exceptions:
+            name = os.path.join(path, name)
+            if os.path.isdir(name):
+                walk_with_symlinks(name, visit, arg)
+
 def remove_stale_bytecode(arg, dirname, names):
     names = map(os.path.normcase, names)
     for name in names:
@@ -598,33 +634,45 @@ def remove_stale_bytecode(arg, dirname, names):
             srcname = name[:-1]
             if srcname not in names:
                 fullname = os.path.join(dirname, name)
-                print "Removing stale bytecode file", fullname
-                os.unlink(fullname)
+                print "Removing stale bytecode file", fullname,
+                try:
+                    os.unlink(fullname)
+                except (OSError, IOError), e:
+                    print ' -->  %s (errno %d)' % (e.strerror, e.errno)
+                else:
+                    print
+
 
 def main(module_filter, test_filter, libdir):
-    if not keepStaleBytecode:
-        os.path.walk(os.curdir, remove_stale_bytecode, None)
-
     global pathinit
-    
-    # Get the log.ini file from the current directory instead of possibly
-    # buried in the build directory.  XXX This isn't perfect because if
-    # log.ini specifies a log file, it'll be relative to the build directory.
-    # Hmm...
-    logini = os.path.abspath('log.ini')
+    global config_file
+
+    configure_logging()
 
     # Initialize the path and cwd
     pathinit = PathInit(build, libdir)
 
-    # Initialize the logging module.
-    import logging.config
-    logging.basicConfig()
-    level = os.getenv("LOGGING")
-    if level:
-        level = int(level)
-    else:
-        level = logging.CRITICAL
-    logging.root.setLevel(level)
+    if not keepStaleBytecode:
+        walk_with_symlinks(pathinit.home, remove_stale_bytecode, None)
+    
+    # Load configuration
+    if config_file:
+        config_file = os.path.realpath(config_file)
+        print "Parsing %s" % config_file
+        import Zope
+        Zope.configure(config_file)
+
+    if not keepStaleBytecode:
+        from App.config import getConfiguration
+        softwarehome = os.path.realpath(getConfiguration().softwarehome)
+        instancehome = os.path.realpath(getConfiguration().instancehome)
+        softwarehome = os.path.normcase(softwarehome)
+        if not softwarehome.startswith(os.path.normcase(instancehome)):
+            walk_with_symlinks(instancehome, remove_stale_bytecode, None)
+
+    # Import Testing package to setup the test ZODB
+    if import_testing:
+        import Testing
 
     files = find_tests(module_filter)
     files.sort()
@@ -650,6 +698,26 @@ def main(module_filter, test_filter, libdir):
         runner(files, test_filter, debug)
 
 
+def configure_logging():
+    """Initialize the logging module."""
+    import logging.config
+
+    # Get the log.ini file from the current directory instead of possibly                                      
+    # buried in the build directory.  XXX This isn't perfect because if
+    # log.ini specifies a log file, it'll be relative to the build directory.
+    # Hmm...
+    logini = os.path.abspath("log.ini")
+
+    if os.path.exists(logini):
+        logging.config.fileConfig(logini)
+    else:
+        logging.basicConfig()
+
+    if os.environ.has_key("LOGGING"):
+        level = int(os.environ["LOGGING"])
+        logging.getLogger().setLevel(level)
+
+
 def process_args(argv=None):
     import getopt
     global module_filter
@@ -670,6 +738,8 @@ def process_args(argv=None):
     global keepStaleBytecode
     global functional
     global test_dir
+    global config_file
+    global import_testing
 
     if argv is None:
         argv = sys.argv
@@ -695,11 +765,14 @@ def process_args(argv=None):
     keepStaleBytecode = 0
     functional = False
     test_dir = None
+    config_file = None
+    import_testing = False
 
     try:
-        opts, args = getopt.getopt(argv[1:], "a:bcdDfg:G:hLmprtTuv",
+        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmprtTuv",
                                    ["all", "help", "libdir=", "times=",
-                                    "keepbytecode", "dir="])
+                                    "keepbytecode", "dir=", 
+                                    "config-file=", "import-testing"])
     except getopt.error, msg:
         print msg
         print "Try `python %s -h' for more information." % argv[0]
@@ -766,6 +839,10 @@ def process_args(argv=None):
                 timesfn = v
         elif k == '--dir':
             test_dir = v
+        elif k == '--config-file' or k == '-C':
+            config_file = v
+        elif k == '--import-testing':
+            import_testing = True
 
     if gcthresh is not None:
         if gcthresh == 0:
