@@ -12,7 +12,7 @@
 ##############################################################################
 """Encapsulation of date/time values"""
 
-__version__='$Revision: 1.85.12.10 $'[11:-2]
+__version__='$Revision: 1.85.12.13 $'[11:-2]
 
 
 import re, math,  DateTimeZone
@@ -462,6 +462,12 @@ def _tzoffset2rfc822zone(seconds):
        _tzoffset() is the negative of what time.localzone and time.altzone is."""
     return "%+03d%02d" % divmod( (seconds/60), 60) 
 
+def _tzoffset2iso8601zone(seconds):
+    """Takes an offset, such as from _tzoffset(), and returns an ISO 8601
+       compliant zone specification. Please note that the result of 
+       _tzoffset() is the negative of what time.localzone and time.altzone is."""
+    return "%+03d:%02d" % divmod( (seconds/60), 60) 
+
 
 class DateTime:
     """DateTime objects represent instants in time and provide
@@ -516,6 +522,16 @@ class DateTime:
     SyntaxError = SyntaxError
 
     def __init__(self,*args, **kw):
+        """Return a new date-time object"""
+
+        try:
+            return self._parse_args(*args, **kw)
+        except (DateError, TimeError, DateTimeError):
+            raise
+        except:
+            raise SyntaxError('Unable to parse %s, %s' % (args, kw))
+
+    def _parse_args(self,*args, **kw):
         """Return a new date-time object
 
         A DateTime object always maintains its value as an absolute
@@ -707,7 +723,17 @@ class DateTime:
             if arg=='':
                 raise SyntaxError, arg
 
-            if isinstance(arg, (unicode, str)) and arg.lower() in self._tzinfo._zidx:
+            if isinstance(arg, DateTime):
+                """ Construct a new DateTime instance from a given DateTime instance """
+                t = arg.timeTime()
+                lt = safelocaltime(t)
+                tz = self.localZone(lt)
+                ms = (t - math.floor(t))
+                s,d = _calcSD(t)
+                yr,mo,dy,hr,mn,sc=lt[:6]
+                sc=sc+ms
+
+            elif isinstance(arg, (unicode, str)) and arg.lower() in self._tzinfo._zidx:
                 # Current time, to be displayed in specified timezone
                 t,tz=time(),self._tzinfo._zmap[arg.lower()]
                 ms=(t-math.floor(t))
@@ -1468,12 +1494,8 @@ class DateTime:
         return millis
 
     def strftime(self, format):
-        # Format the date/time using the *current timezone representation*.
-        diff = _tzoffset(self._tz, self._t)
-        format = re.sub('(^\|[^%])%z',
-                               '\\1%+05d' % (diff / 36),
-                               format)
-        return strftime(format, safegmtime(self.timeTime() + diff))
+        # Format the date/time using the *local timezone representation*.
+        return strftime(format, safelocaltime(self.timeTime()))
 
     # General formats from previous DateTime
     def Date(self):
@@ -1571,7 +1593,9 @@ class DateTime:
 
 
     def ISO(self):
-        """Return the object in ISO standard format
+        """Return the object in ISO standard format. Note:
+        this is *not* ISO 8601-format! See the ISO8601 and
+        HTML4 methods below for ISO 8601-compliant output
 
         Dates are output as: YYYY-MM-DD HH:MM:SS
         """
@@ -1579,6 +1603,24 @@ class DateTime:
             self._year, self._month, self._day,
             self._hour, self._minute, self._second)
 
+    def ISO8601(self):
+        """Return the object in ISO 8601-compatible format containing
+        the date, time with seconds-precision and the time zone 
+        identifier - see http://www.w3.org/TR/NOTE-datetime
+
+        Dates are output as: YYYY-MM-DDTHH:MM:SSTZD
+            T is a literal character.
+            TZD is Time Zone Designator, format +HH:MM or -HH:MM
+
+        The HTML4 method below offers the same formatting, but converts
+        to UTC before returning the value and sets the TZD "Z"
+        """
+        tzoffset = _tzoffset2iso8601zone(_tzoffset(self._tz, self._t))
+
+        return  "%0.4d-%0.2d-%0.2dT%0.2d:%0.2d:%0.2d%s" % (
+            self._year, self._month, self._day,
+            self._hour, self._minute, self._second, tzoffset)
+
     def HTML4(self):
         """Return the object in the format used in the HTML4.0 specification,
         one of the standard forms in ISO8601.  See
