@@ -11,15 +11,16 @@
 #
 ##############################################################################
 __doc__="""Copy interface"""
-__version__='$Revision: 1.85.2.6 $'[11:-2]
+__version__='$Revision: 1.85.2.9 $'[11:-2]
 
-import sys,  Globals, Moniker, tempfile, ExtensionClass
+import re, sys, Globals, Moniker, tempfile, ExtensionClass
 from marshal import loads, dumps
 from urllib import quote, unquote
 from zlib import compress, decompress
 
 from App.Dialogs import MessageDialog
 from AccessControl import getSecurityManager
+from AccessControl.Permissions import delete_objects as DeleteObjects
 from Acquisition import aq_base, aq_inner, aq_parent
 from zExceptions import Unauthorized, BadRequest
 from webdav.Lockable import ResourceLockedError
@@ -27,6 +28,8 @@ from cgi import escape
 
 CopyError='Copy Error'
 
+copy_re = re.compile('^copy([0-9]*)_of_(.*)')
+
 _marker=[]
 class CopyContainer(ExtensionClass.Base):
     """Interface for containerish objects which allow cut/copy/paste"""
@@ -115,10 +118,13 @@ class CopyContainer(ExtensionClass.Base):
         # Allow containers to override the generation of
         # object copy id by attempting to call its _get_id
         # method, if it exists.
-        n=0
-        if (len(id) > 8) and (id[8:]=='copy_of_'):
-            n=1
-        orig_id=id
+        match = copy_re.match(id)
+        if match:
+            n = int(match.group(1) or '1')
+            orig_id = match.group(2)
+        else:
+            n = 0
+            orig_id = id
         while 1:
             if self._getOb(id, None) is None:
                 return id
@@ -151,7 +157,7 @@ class CopyContainer(ExtensionClass.Base):
             m = Moniker.loadMoniker(mdata)
             try: ob = m.bind(app)
             except: raise CopyError, eNotFound
-            self._verifyObjectPaste(ob)
+            self._verifyObjectPaste(ob, validate_src=op+1)
             oblist.append(ob)
 
         if op==0:
@@ -378,13 +384,23 @@ class CopyContainer(ExtensionClass.Base):
                   action = 'manage_main')
 
             if validate_src:
+
+                sm = getSecurityManager()
+
                 # Ensure the user is allowed to access the object on the
                 # clipboard.
-                try:    parent = aq_parent(aq_inner(object))
-                except: parent = None
-                if not getSecurityManager().validate(None,parent,None,object):
+                try:
+                    parent = aq_parent(aq_inner(object))
+                except:
+                    parent = None
+
+                if not sm.validate(None,parent,None,object):
                     raise Unauthorized, absattr(object.id)
 
+                if validate_src == 2: # moving
+                    if not sm.checkPermission(DeleteObjects, parent):
+                        raise Unauthorized, 'Delete not allowed.'
+
         else: # /if method_name
             raise CopyError, MessageDialog(
                   title   = 'Not Supported',
