@@ -12,7 +12,7 @@
 ##############################################################################
 """Image object"""
 
-__version__='$Revision: 1.145.2.7 $'[11:-2]
+__version__='$Revision: 1.145.2.10 $'[11:-2]
 
 import Globals, struct
 from OFS.content_types import guess_content_type
@@ -76,6 +76,7 @@ class File(Persistent, Implicit, PropertyManager,
 
     precondition=''
     size=None
+    alt=''
 
     manage_editForm  =DTMLFile('dtml/fileEdit',globals(),
                                Kind='File',kind='file')
@@ -113,6 +114,7 @@ class File(Persistent, Implicit, PropertyManager,
 
 
     _properties=({'id':'title', 'type': 'string'},
+                 {'id':'alt', 'type':'string'},
                  {'id':'content_type', 'type':'string'},
                  )
 
@@ -477,8 +479,10 @@ class File(Persistent, Implicit, PropertyManager,
         if type(file) is StringType:
             size=len(file)
             if size < n: return file, size
-            return Pdata(file), size
-        elif isinstance(file, FileUpload) and not file:
+            # Big string: cut it into smaller chunks
+            file = StringIO(file)
+
+        if isinstance(file, FileUpload) and not file:
             raise ValueError, 'File not specified'
 
         if hasattr(file, '__class__') and file.__class__ is Pdata:
@@ -791,7 +795,7 @@ class Image(File):
         result='<img src="%s"' % (self.absolute_url())
 
         if alt is None:
-            alt=getattr(self, 'title', '')
+            alt=getattr(self, 'alt', '')
         result = '%s alt="%s"' % (result, escape(alt, 1))
 
         if title is None:
@@ -804,8 +808,9 @@ class Image(File):
         if width:
             result = '%s width="%s"' % (result, width)
 
-        if not 'border' in [ x.lower() for x in  args.keys()]:
-            result = '%s border="0"' % result
+         # Collector #1557
+#        if not 'border' in [ x.lower() for x in  args.keys()]:
+#            result = '%s border="0"' % result
 
         if css_class is not None:
             result = '%s class="%s"' % (result, css_class)
