@@ -12,9 +12,9 @@
 ##############################################################################
 __doc__="""Object Manager
 
-$Id: ObjectManager.py,v 1.158 2002/08/14 21:42:56 mj Exp $"""
+$Id: ObjectManager.py,v 1.163.12.11 2004/06/25 09:28:09 ctheune Exp $"""
 
-__version__='$Revision: 1.158 $'[11:-2]
+__version__='$Revision: 1.163.12.11 $'[11:-2]
 
 import App.Management, Acquisition, Globals, CopySupport, Products
 import os, App.FactoryDispatcher, re, Products
@@ -27,12 +27,16 @@ from webdav.NullResource import NullResource
 from webdav.Collection import Collection
 from Acquisition import aq_base
 from AccessControl.SecurityInfo import ClassSecurityInfo
+from webdav.Lockable import ResourceLockedError
+from ZODB.POSException import ConflictError
 from urllib import quote
 from cStringIO import StringIO
 import marshal
 import App.Common
+from App.config import getConfiguration
 from AccessControl import getSecurityManager
 from zLOG import LOG, ERROR
+from zExceptions import BadRequest
 import sys,fnmatch,copy
 from cgi import escape
 from types import StringType, UnicodeType
@@ -44,8 +48,6 @@ customImporters={
 
 bad_id=re.compile(r'[^a-zA-Z0-9-_~,.$\(\)# ]').search #TS
 
-BadRequestException = 'Bad Request'
-
 def checkValidId(self, id, allow_dup=0):
     # If allow_dup is false, an error will be raised if an object
     # with the given id already exists. If allow_dup is true,
@@ -55,16 +57,18 @@ def checkValidId(self, id, allow_dup=0):
 
     if not id or not isinstance(id, StringType):
         if isinstance(id, UnicodeType): id = escape(id)
-        raise BadRequestException, ('Empty or invalid id specified', id)
+        raise BadRequest, ('Empty or invalid id specified', id)
     if bad_id(id) is not None:
-        raise BadRequestException, (
+        raise BadRequest, (
             'The id "%s" contains characters illegal in URLs.' % escape(id))
-    if id[0]=='_': raise BadRequestException, (
-        'The id "%s" is invalid - it begins with an underscore.'  % id)
-    if id[:3]=='aq_': raise BadRequestException, (
-        'The id "%s" is invalid - it begins with "aq_".'  % id)
-    if id[-2:]=='__': raise BadRequestException, (
-        'The id "%s" is invalid - it ends with two underscores.'  % id)
+    if id in ('.', '..'): raise BadRequest, (
+        'The id "%s" is invalid because it is not traversable.' % id)
+    if id.startswith('_'): raise BadRequest, (
+        'The id "%s" is invalid because it begins with an underscore.' % id)
+    if id.startswith('aq_'): raise BadRequest, (
+        'The id "%s" is invalid because it begins with "aq_".' % id)
+    if id.endswith('__'): raise BadRequest, (
+        'The id "%s" is invalid because it ends with two underscores.' % id)
     if not allow_dup:
         obj = getattr(self, id, None)
         if obj is not None:
@@ -74,21 +78,21 @@ def checkValidId(self, id, allow_dup=0):
             if hasattr(aq_base(self), id):
                 # The object is located in this ObjectManager.
                 if not flags & REPLACEABLE:
-                    raise BadRequestException, ('The id "%s" is invalid--'
-                                          'it is already in use.' % id)
+                    raise BadRequest, (
+                        'The id "%s" is invalid - it is already in use.' % id)
                 # else the object is replaceable even if the UNIQUE
                 # flag is set.
             elif flags & UNIQUE:
-                raise BadRequestException, ('The id "%s" is reserved.' % id)
+                raise BadRequest, ('The id "%s" is reserved.' % id)
     if id == 'REQUEST':
-        raise BadRequestException, 'REQUEST is a reserved name.'
+        raise BadRequest, 'REQUEST is a reserved name.'
     if '/' in id:
-        raise BadRequestException, (
-            'The id "%s" contains characters illegal in URLs.' % id
-            )
+        raise BadRequest, (
+            'The id "%s" contains characters illegal in URLs.' % id)
 
-class BeforeDeleteException( Exception ): pass # raise to veto deletion
-class BreakoutException ( Exception ): pass  # raised to break out of loops
+class BeforeDeleteException(Exception): pass # raise to veto deletion
+class BreakoutException(Exception): pass  # raised to break out of loops
+BadRequestException = BadRequest
 
 _marker=[]
 class ObjectManager(
@@ -265,9 +269,9 @@ class ObjectManager(
                 if object.__ac_local_roles__ is None:
                     user=getSecurityManager().getUser()
                     if user is not None:
-                        name=user.getUserName()
-                        if name != 'Anonymous User':
-                            object.manage_setLocalRoles(name, ['Owner'])
+                        userid=user.getId()
+                        if userid is not None:
+                            object.manage_setLocalRoles(userid, ['Owner'])
 
         object.manage_afterAdd(object, self)
         return id
@@ -297,6 +301,8 @@ class ObjectManager(
                     object.manage_beforeDelete(item, container)
             except BeforeDeleteException, ob:
                 raise
+            except ConflictError:
+                raise
             except:
                 LOG('Zope',ERROR,'manage_beforeDelete() threw',
                     error=sys.exc_info())
@@ -304,11 +310,16 @@ class ObjectManager(
             if s is None: object._p_deactivate()
 
     def _delObject(self, id, dp=1):
+        # dp is here for backwards compatibility as it once
+        # allowed to override the use of the delete protocol.
+        # See http://collector.zope.org/Zope/1377 
         object=self._getOb(id)
         try:
             object.manage_beforeDelete(object, self)
         except BeforeDeleteException, ob:
             raise
+        except ConflictError:
+            raise
         except:
             LOG('Zope',ERROR,'manage_beforeDelete() threw',
                 error=sys.exc_info())
@@ -336,23 +347,21 @@ class ObjectManager(
                 if ob['meta_type'] in spec:
                     set.append(ob['id'])
             return set
-        return map(lambda i: i['id'], self._objects)
+        return [ o['id']  for o in self._objects ]
+
 
     def objectValues(self, spec=None):
         # Returns a list of actual subobjects of the current object.
         # If 'spec' is specified, returns only objects whose meta_type
         # match 'spec'.
-        return map(self._getOb, self.objectIds(spec))
+        return [ self._getOb(id) for id in self.objectIds(spec) ]
+
 
     def objectItems(self, spec=None):
         # Returns a list of (id, subobject) tuples of the current object.
         # If 'spec' is specified, returns only objects whose meta_type match
         # 'spec'
-        r=[]
-        a=r.append
-        g=self._getOb
-        for id in self.objectIds(spec): a((id, g(id)))
-        return r
+        return [ (id, self._getOb(id)) for id in self.objectIds(spec) ]
 
     def objectMap(self):
         # Return a tuple of mappings containing subobject meta-data
@@ -442,8 +451,13 @@ class ObjectManager(
         while ids:
             id=ids[-1]
             v=self._getOb(id, self)
+
+            if v.wl_isLocked():
+                raise ResourceLockedError, (
+                    'Object "%s" is locked via WebDAV' % v.getId())
+
             if v is self:
-                raise 'BadRequest', '%s does not exist' % escape(ids[-1])
+                raise BadRequest, '%s does not exist' % escape(ids[-1])
             self._delObject(id)
             del ids[-1]
         if REQUEST is not None:
@@ -495,7 +509,8 @@ class ObjectManager(
                                    'inline;filename=%s.%s' % (id, suffix))
             return f.getvalue()
 
-        f = os.path.join(CLIENT_HOME, '%s.%s' % (id, suffix))
+        cfg = getConfiguration()
+        f = os.path.join(cfg.clienthome, '%s.%s' % (id, suffix))
         if toxml:
             XMLExportImport.exportXML(ob._p_jar, ob._p_oid, f)
         else:
@@ -504,7 +519,7 @@ class ObjectManager(
         if REQUEST is not None:
             return self.manage_main(self, REQUEST,
                 manage_tabs_message=
-                '<em>%s</em> sucessfully exported to <em>%s</em>' % (id,f),
+                '<em>%s</em> successfully exported to <em>%s</em>' % (id,f),
                 title = 'Object exported')
 
 
@@ -514,25 +529,24 @@ class ObjectManager(
         """Import an object from a file"""
         dirname, file=os.path.split(file)
         if dirname:
-            raise BadRequestException, 'Invalid file name %s' % escape(file)
+            raise BadRequest, 'Invalid file name %s' % escape(file)
 
-        instance_home = INSTANCE_HOME
-        zope_home = ZOPE_HOME
-
-        for impath in (instance_home, zope_home):
+        cfg = getConfiguration()
+        for impath in (cfg.instancehome, cfg.zopehome):
             filepath = os.path.join(impath, 'import', file)
             if os.path.exists(filepath):
                 break
         else:
-            raise BadRequestException, 'File does not exist: %s' % escape(file)
+            raise BadRequest, 'File does not exist: %s' % escape(file)
 
         self._importObjectFromFile(filepath, verify=not not REQUEST,
                                    set_owner=set_owner)
 
         if REQUEST is not None:
-            return self.manage_main(self, REQUEST,
-                manage_tabs_message='<em>%s</em> sucessfully imported' % id,
-                title = 'Object imported',
+            return self.manage_main(
+                self, REQUEST,
+                manage_tabs_message='<em>%s</em> successfully imported' % id,
+                title='Object imported',
                 update_menu=1)
 
     def _importObjectFromFile(self, filepath, verify=1, set_owner=1):
@@ -579,17 +593,13 @@ class ObjectManager(
             all_files = copy.copy(files)
             for f in files:
                 if f[1].meta_type == "Folder":
-                    all_files.extend(findChilds(f[1]))
+                    all_files.extend(findChildren(f[1]))
                 else:
                     all_files.append(f)
 
             files = all_files
 
-        try:
-            files.sort()
-        except AttributeError:
-            files=list(files)
-            files.sort()
+        files = list(files)
 
         # Perform globbing on list of files (ajung)
 
@@ -597,21 +607,23 @@ class ObjectManager(
         if globbing :
             files = filter(lambda x,g=globbing: fnmatch.fnmatch(x[0],g) , files)
 
-        try:
-            files.sort()
-        except AttributeError:
-            files=list(files)
-            files.sort()
+        files.sort()
 
         if not (hasattr(self,'isTopLevelPrincipiaApplicationObject') and
                 self.isTopLevelPrincipiaApplicationObject):
             files.insert(0,('..',self.aq_parent))
+        files.insert(0, ('.', self))
         for k,v in files:
             # Note that we have to tolerate failure here, because
             # Broken objects won't stat correctly. If an object fails
-            # to be able to stat itself, we will ignore it.
-            try:    stat=marshal.loads(v.manage_FTPstat(REQUEST))
-            except: stat=None
+            # to be able to stat itself, we will ignore it, but log
+            # the error.
+            try:
+                stat=marshal.loads(v.manage_FTPstat(REQUEST))
+            except:
+                LOG("FTP", ERROR, "Failed to stat file '%s'" % k,
+                    error=sys.exc_info())
+                stat=None
             if stat is not None:
                 out=out+((k,stat),)
         return marshal.dumps(out)
@@ -640,7 +652,6 @@ class ObjectManager(
                 break
         return marshal.dumps((mode,0,0,1,owner,group,0,mtime,mtime,mtime))
 
-
     def __getitem__(self, key):
         v=self._getOb(key, None)
         if v is not None: return v
@@ -651,7 +662,7 @@ class ObjectManager(
                 return NullResource(self, key, request).__of__(self)
         raise KeyError, key
 
-def findChilds(obj,dirname=''):
+def findChildren(obj,dirname=''):
     """ recursive walk through the object hierarchy to
     find all childs of an object (ajung)
     """
@@ -659,7 +670,7 @@ def findChilds(obj,dirname=''):
     lst =[]
     for name,child in obj.objectItems():
         if child.meta_type=="Folder":
-            lst.extend(findChilds(child,dirname+ obj.id + '/'))
+            lst.extend(findChildren(child,dirname+ obj.id + '/'))
         else:
             lst.append( (dirname + obj.id + "/" + name,child) )
 
