@@ -12,8 +12,8 @@
 ##############################################################################
 __doc__='''Search Interface Wizard
 
-$Id: Search.py,v 1.20.68.2 2004/01/08 23:33:58 tseaver Exp $'''
-__version__='$Revision: 1.20.68.2 $'[11:-2]
+$Id: Search.py,v 1.20.68.3 2004/11/14 21:35:03 efge Exp $'''
+__version__='$Revision: 1.20.68.3 $'[11:-2]
 
 from Globals import DTMLFile
 from Aqueduct import custom_default_report, custom_default_zpt_report, nicify, Args
@@ -173,7 +173,7 @@ def default_input_form(arguments,action='query',
                         lambda a:
                         ('<tr><th>%s</th>\n'
                          '    <td><input name="%s"\n'
-                         '               width=30 value="%s">'
+                         '               size="30" value="%s">'
                          '</td></tr>'
                          % (nicify(a[0]),
                             (
@@ -223,7 +223,7 @@ def default_input_zpt_form(arguments,action='query',
                         lambda a:
                         ('<tr><th>%s</th>\n'
                          '    <td><input name="%s"\n'
-                         '               width=30 value="%s">'
+                         '               size="30" value="%s">'
                          '</td></tr>'
                          % (nicify(a[0]),
                             (
