@@ -156,7 +156,10 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         self.thr_async = 0
         self.trigger = None
         self._prepare_async()
-        self._map = {self._fileno: self}
+        # The singleton dict is used in synchronous mode when a method
+        # needs to call into asyncore to try to force some I/O to occur.
+        # The singleton dict is a socket map containing only this object.
+        self._singleton = {self._fileno: self}
         # msgid_lock guards access to msgid
         self.msgid_lock = threading.Lock()
         # replies_cond is used to block when a synchronous call is
@@ -182,7 +185,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     def close(self):
         if self.closed:
             return
-        self._map.clear()
+        self._singleton.clear()
         self.closed = 1
         self.close_trigger()
         self.__super_close()
@@ -383,7 +386,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         if self.closed:
             raise DisconnectedError()
         msgid = self.send_call(method, args, 0)
-        asyncore.poll(0.01, self._map)
+        asyncore.poll(0.01, self._singleton)
         return msgid
 
     def _deferred_wait(self, msgid):
@@ -471,7 +474,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                             if __debug__:
                                 self.log("wait(%d): asyncore.poll(%s)" %
                                          (msgid, delay), level=zLOG.TRACE)
-                            asyncore.poll(delay, self._map)
+                            asyncore.poll(delay, self._singleton)
                             if delay < 1.0:
                                 delay += delay
                         except select.error, err:
@@ -497,7 +500,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         if self.is_async():
             self._pull_trigger()
         else:
-            asyncore.poll(0.0, self._map)
+            asyncore.poll(0.0, self._singleton)
 
     def pending(self, timeout=0):
         """Invoke mainloop until any pending messages are handled."""
