@@ -13,17 +13,7 @@
 ##############################################################################
 """Database connection support
 
-$Id: Connection.py,v 1.98.4.6 2004/05/21 16:14:04 tim_one Exp $"""
-
-from cPickleCache import PickleCache
-from POSException import ConflictError, ReadConflictError, TransactionError
-from ExtensionClass import Base
-import ExportImport, TmpStore
-from zLOG import LOG, ERROR, BLATHER, WARNING
-from coptimizations import new_persistent_id
-from ConflictResolution import ResolvedSerial
-from Transaction import Transaction, get_transaction
-from ZODB.utils import oid_repr
+$Id: Connection.py,v 1.98.4.10 2004/09/16 23:56:18 tim_one Exp $"""
 
 from cPickle import Unpickler, Pickler
 from cStringIO import StringIO
@@ -32,6 +22,17 @@ import threading
 from time import time
 from types import StringType, ClassType
 
+from cPickleCache import PickleCache
+from POSException import ConflictError, ReadConflictError, TransactionError
+from POSException import ConnectionStateError
+from ExtensionClass import Base
+import ExportImport, TmpStore
+from zLOG import LOG, ERROR, BLATHER, WARNING
+from coptimizations import new_persistent_id
+from ConflictResolution import ResolvedSerial
+from Transaction import Transaction, get_transaction
+from ZODB.utils import oid_repr
+
 global_code_timestamp = 0
 
 def updateCodeTimestamp():
@@ -53,9 +54,9 @@ class Connection(ExportImport.ExportImport, object):
 
     The Connection manages movement of objects in and out of object storage.
     """
-    _tmp=None
-    _debug_info=()
-    _opened=None
+    _tmp = None
+    _debug_info = ()
+    _opened = None
     _code_timestamp = 0
     _transaction = None
 
@@ -96,8 +97,29 @@ class Connection(ExportImport.ExportImport, object):
         self._inv_lock = threading.Lock()
         self._invalidated = d = {}
         self._invalid = d.has_key
+
+        # We intend to prevent committing a transaction in which
+        # ReadConflictError occurs.   _conflicts is the set of oids that
+        # experienced ReadConflictError.  Any time we raise ReadConflictError,
+        # the oid should be added to this set, and we should be sure that the
+        # object is registered with the transaction.  Because it's registered,
+        # Transaction.commit() will call Connection.commit() for it, and the
+        # latter will raise ReadConflictError again (because the oid is in
+        # _conflicts).
         self._conflicts = {}
 
+        # We want to raise ConnectionStateError if an attempt to close is
+        # made while modifications are pending.  This is easier in ZODB
+        # 3.3.  While it's not obvious, self._tmp is non-None iff a
+        # committed subtransaction is pending.  For "top level" pending
+        # changes, we set _object_registered to True in register(), and
+        # back to False in abort() and _flush_invalidations().  The latter
+        # is an odd choice, but is common to most ways of clearing pending
+        # top-level change (tpc_finish, tpc_abort, sync, _setDB).  abort()
+        # is also a bit peculiar, but it's the only Connection method
+        # Transaction.abort() calls, so there's not much choice.
+        self._object_registered = False
+
     def getTransaction(self):
         t = self._transaction
         if t is None:
@@ -258,6 +280,8 @@ class Connection(ExportImport.ExportImport, object):
         else:
             assert object._p_oid is not None
             self._cache.invalidate(object._p_oid)
+            self._object_registered = False
+            self._conflicts.clear()
 
     def cacheFullSweep(self, dt=0):
         self._cache.full_sweep(dt)
@@ -274,6 +298,19 @@ class Connection(ExportImport.ExportImport, object):
         self.__onCloseCallbacks.append(f)
 
     def close(self):
+        if self._object_registered:
+            # We're currently joined to a transaction.  This spelling
+            # doesn't make much sense here, but I want to keep this
+            # exception the same as in ZODB 3.3 (where this spelling
+            # does make sense).
+            raise ConnectionStateError("Cannot close a connection joined to "
+                                       "a transaction")
+
+        if self._tmp is not None:
+            # There are no direct modifications pending, but a subtransaction
+            # is pending.
+            raise ConnectionStateError("Cannot close a connection with a "
+                                       "pending subtransaction")
         if self._incrgc is not None:
             self._incrgc() # This is a good time to do some GC
 
@@ -309,7 +346,6 @@ class Connection(ExportImport.ExportImport, object):
 
         oid = object._p_oid
         if self._conflicts.has_key(oid):
-            self.getTransaction().register(object)
             raise ReadConflictError(object=object)
 
         invalid = self._invalid
@@ -519,6 +555,7 @@ class Connection(ExportImport.ExportImport, object):
             self._inv_lock.release()
         # Now is a good time to collect some garbage
         self._cache.incrgc()
+        self._object_registered = False
 
     def modifiedInVersion(self, oid):
         try: return self._db.modifiedInVersion(oid)
@@ -535,6 +572,7 @@ class Connection(ExportImport.ExportImport, object):
         # XXX Figure out why this assert causes test failures
         # assert object._p_oid is not None
         self.getTransaction().register(object)
+        self._object_registered = True
 
     def root(self):
         return self['\0\0\0\0\0\0\0\0']
@@ -622,6 +660,7 @@ class Connection(ExportImport.ExportImport, object):
                 self._inv_lock.release()
         else:
             self.getTransaction().register(obj)
+            self._conflicts[obj._p_oid] = 1
             raise ReadConflictError(object=obj)
 
     def _ignore_conflict(self, oid):
