@@ -42,6 +42,12 @@ class BatchTests(TestCase):
         assert b.sequence_length == 8
         assert len(b.next) == 3
 
+    def testLengthEqualsSizePlusOrphans(self):
+        '''Test limit case where batch length is equal to size + orphans'''
+        for bsize, length in ((12,11), (13,12), (14,13), (15,10)):
+            b = Batch(range(bsize), size=10, start=1, end=0, orphan=3, overlap=0)
+            assert length == b.length
+    
 def test_suite():
     return makeSuite(BatchTests)
 
